@@ -177,6 +177,28 @@ class RawExecutionMatrix(object):
 
     #def raw_delete_row(self):
 
+    def clear_cells_to_value(self, value):
+        """ clear the matrix values to a given value
+        :param value: cell value type
+        :return: nothing
+            
+        Example:
+        >>> nc = ['a', 'b', 'c']
+        >>> mat = ExecutionMatrix(non_key_col_list=nc)
+        >>> mat.add_row_by_key('k', [1, 2, 3])
+        >>> mat.add_row_by_key('r', [3, 2, 3])
+        >>> mat.add_row_by_key('w', [1, 0, 3])
+        >>> mat.clear_cells_to_value(0)
+        >>> mat._get_key_values_dict(['k']) == {'k': {'a':0,'b':0,'c':0}}
+        True
+        >>> mat._get_key_values_dict(['r']) == {'r': {'a':0,'b':0,'c':0}}
+        True
+        >>> mat._get_key_values_dict(['w']) == {'w': {'a':0,'b':0,'c':0}}
+        True
+        """
+        self.dataframe[list(self.get_nonkey_colname_list())] = value
+        
+    
     def add_row_by_key(self, key, values, serialize=True):
         """ add a row to the matrix
         :param key: The key to add
@@ -402,7 +424,7 @@ class RawExecutionMatrix(object):
             small_df = self.extract_by_rowkey(row_key_list).dataframe
             for _, row in small_df.iterrows():
                 result[row[self.key_column_name]] = \
-                                        [x for x in self.non_key_col_list \
+                                [sys.intern(x) for x in self.non_key_col_list \
                                         if self.is_active_cell_func(row[x])]
 
         return result
@@ -431,9 +453,9 @@ class RawExecutionMatrix(object):
         if len(non_key_col_list) > 0:
             small_df = self.extract_by_column(non_key_col_list).dataframe
             for col in non_key_col_list:
-                result[col] = list(small_df.loc[\
+                result[col] = [sys.intern(x) for x in small_df.loc[\
                                 small_df[col].apply(self.is_active_cell_func)]\
-                                                        [self.key_column_name])
+                                                        [self.key_column_name]]
 
         return result
     #~ def query_active_rows_of_columns()
@@ -462,7 +484,7 @@ class RawExecutionMatrix(object):
             small_df = self.extract_by_rowkey(row_key_list).dataframe
             for _, row in small_df.iterrows():
                 result[row[self.key_column_name]] = \
-                                        [x for x in self.non_key_col_list \
+                                [sys.intern(x) for x in self.non_key_col_list \
                                         if self.is_inactive_cell_func(row[x])]
 
         return result
@@ -491,9 +513,9 @@ class RawExecutionMatrix(object):
         if len(non_key_col_list) > 0:
             small_df = self.extract_by_column(non_key_col_list).dataframe
             for col in non_key_col_list:
-                result[col] = list(small_df.loc[\
+                result[col] = [sys.intern(x) for x in small_df.loc[\
                             small_df[col].apply(self.is_inactive_cell_func)]\
-                                                        [self.key_column_name])
+                                                        [self.key_column_name]]
 
         return result
     #~ def query_inactive_rows_of_columns()
@@ -522,7 +544,7 @@ class RawExecutionMatrix(object):
             small_df = self.extract_by_rowkey(row_key_list).dataframe
             for _, row in small_df.iterrows():
                 result[row[self.key_column_name]] = \
-                                        [x for x in self.non_key_col_list \
+                                [sys.intern(x) for x in self.non_key_col_list \
                                         if self.is_uncertain_cell_func(row[x])]
 
         return result
@@ -551,9 +573,9 @@ class RawExecutionMatrix(object):
         if len(non_key_col_list) > 0:
             small_df = self.extract_by_column(non_key_col_list).dataframe
             for col in non_key_col_list:
-                result[col] = list(small_df.loc[\
+                result[col] = [sys.intern(x) for x in small_df.loc[\
                             small_df[col].apply(self.is_uncertain_cell_func)]\
-                                                        [self.key_column_name])
+                                                        [self.key_column_name]]
 
         return result
     #~ def query_uncertain_rows_of_columns()
@@ -758,9 +780,21 @@ class OutputLogData(object):
         if self.filename is None or not os.path.isfile(self.filename):
             self.data = {}
         else:
-            self.data = common_fs.loadJSON(self.filename)
+            self.data = {objective: self._mem_optimize_sub_dat(test2dat) \
+                                for objective, test2dat in \
+                                    common_fs.loadJSON(self.filename).items()}
     #~ def __init__()
 
+    @classmethod
+    def _mem_optimize_sub_dat(cls, test2dat):
+        res = {}
+        for t, dat in test2dat.items():
+            t = sys.intern(t)
+            dat[cls.OUTLOG_HASH] = sys.intern(dat[cls.OUTLOG_HASH])
+            res[t] = dat
+        return res
+    #~ def _mem_optimize_sub_dat()
+
     def is_empty(self):
         return len(self.data) == 0
     #~ def is_empty()
@@ -803,9 +837,11 @@ class OutputLogData(object):
                                             "Some values are existing, "
                                             "do you confirm their override?"),\
                              "Existing values were not overriden", __file__)
-            self.data[objective].update(data_dict[objective])
+            self.data[objective].update(self._mem_optimize_sub_dat(\
+                                                        data_dict[objective]))
         for objective in onlynew_objective:
-            self.data[objective] = copy.deepcopy(data_dict[objective])
+            self.data[objective] = self._mem_optimize_sub_dat(\
+                                                        data_dict[objective])
         if serialize:
             self.serialize()
     #~ def add_data ()
