@@ -21,11 +21,27 @@ from muteria.drivers.criteria.base_testcriteriatool import BaseCriteriaTool
 from muteria.drivers.criteria import TestCriteria
 from muteria.drivers import DriversUtils
 
+from muteria.drivers.criteria.tools_by_languages.c.gcov.driver_config \
+                                                        import DriverConfigGCov
+
+
 ERROR_HANDLER = common_mix.ErrorHandler
 
 class CriteriaToolGCov(BaseCriteriaTool):
     def __init__(self, *args, **kwargs):
         BaseCriteriaTool.__init__(self, *args, **kwargs)
+
+        self.driver_config = None
+        if self.config.get_tool_user_custom() is not None:
+            self.driver_config = \
+                            self.config.get_tool_user_custom().DRIVER_CONFIG
+        if self.driver_config is None:
+            self.driver_config = DriverConfigGCov()
+        else:
+            ERROR_HANDLER.assert_true(isinstance(self.driver_config, \
+                                        DriverConfigGCov),\
+                                            "invalid driver config", __file__)
+
         self.instrumentation_details = os.path.join(\
                     self.instrumented_code_storage_dir, '.instru.meta.json')
         self.gcov_files_list_filename = "gcov_files.json"
@@ -46,23 +62,25 @@ class CriteriaToolGCov(BaseCriteriaTool):
     #~ def __init__()
 
     def _get_gcov_list(self):
-        gcov_files = []
-        for root, dirs, files in os.walk(self.gc_files_dir):
-            for file_ in files:
-                if file_.endswith('.gcov'):
-                    gcov_files.append(os.path.join(root, file_))
+        gcov_files = self._recursive_list_files(self.gc_files_dir, '.gcov')
         return gcov_files
     #~ def _get_gcov_list()
 
     def _get_gcda_list(self):
-        gcda_files = []
-        for root, dirs, files in os.walk(self.gc_files_dir):
-            for file_ in files:
-                if file_.endswith('.gcda'):
-                    gcda_files.append(os.path.join(root, file_))
+        gcda_files = self._recursive_list_files(self.gc_files_dir, '.gcda')
         return gcda_files
     #~ def _get_gcda_list()
 
+    @staticmethod
+    def _recursive_list_files(topdir, file_suffix):
+        filtered_files = []
+        for root, dirs, files in os.walk(topdir):
+            for file_ in files:
+                if file_.endswith(file_suffix):
+                    filtered_files.append(os.path.join(root, file_))
+        return filtered_files
+    #~ def _recursive_list_files()
+
     class InstrumentCallbackObject(DefaultCallbackObject):
         def after_command(self):
             if self.op_retval != common_mix.GlobalConstants.COMMAND_SUCCESS:
@@ -78,6 +96,22 @@ class CriteriaToolGCov(BaseCriteriaTool):
                 ERROR_HANDLER.assert_true(os.path.isfile(abs_gcno), \
                                     "gcno file missing after build", __file__)
                 shutil.copy2(abs_gcno, os.path.join(gc_files_dir, gcno_file))
+                # FIXME: For now, we store all gcda in th same directory. 
+                # FIXME: To ensure that coverage is generated with gcov when 
+                # FIXME: The source is not in rootdir (example of coreutils...)
+                gcno_dir, gcno_base = os.path.split(\
+                                                os.path.normpath(gcno_file))
+                gcno_in_top = os.path.join(gc_files_dir, gcno_base)
+                #logging.debug("gcno_instr: '{}' '{}' '{}'".format(gcno_file, \
+                                                    #gcno_dir, gcno_in_top))
+                if gcno_dir != '':
+                    # The gcno file must not be in the top gcno_gcda folder
+                    ERROR_HANDLER.assert_true(not os.path.isfile(gcno_in_top),\
+                            "Overriding of gcno ({}). Source name clash?"\
+                                                .format(gcno_file), __file__)
+                    shutil.copy2(abs_gcno, gcno_in_top)
+                #~ FIXME end
+
             self._copy_from_repo(rel_path_map)
             return DefaultCallbackObject.after_command(self)
         #~ def after_command()
@@ -121,27 +155,35 @@ class CriteriaToolGCov(BaseCriteriaTool):
     #~ def _get_separated_instrumentation_criteria()
 
     def get_instrumented_executable_paths_map(self, enabled_criteria):
-        # use wrapper if gdb is installed
-        using_gdb_wrapper = DriversUtils.check_tool(prog='gdb', \
+        using_gdb_wrapper = self.driver_config.get_use_gdb_wrapper()
+        if using_gdb_wrapper:
+            # use wrapper if gdb is installed
+            using_gdb_wrapper = DriversUtils.check_tool(prog='gdb', \
                                                     args_list=['--version'], \
                                                     expected_exit_codes=[0])
-        if using_gdb_wrapper:
-            # XXX: Docker has issues running programs in Docker, 
-            # unless the container is ran with the following arguments:
-            #
-            # docker run --cap-add=SYS_PTRACE \
-            #               --security-opt seccomp=unconfined ...
-            #
-            # We check that it is fine by testing on echo
-            ret, _, _ = DriversUtils.execute_and_get_retcode_out_err(
+            
+            if using_gdb_wrapper:
+                # XXX: Docker has issues running programs in Docker, 
+                # unless the container is ran with the following arguments:
+                #
+                # docker run --cap-add=SYS_PTRACE \
+                #               --security-opt seccomp=unconfined ...
+                #
+                # We check that it is fine by testing on echo
+                ret, o_e, _ = DriversUtils.execute_and_get_retcode_out_err(
                                         prog='gdb', \
                                         args_list=['--batch-silent', \
                                                     '--quiet', 
                                                     '--return-child-result', 
-                                                    '-ex', '"run"', 
+                                                    '-ex', 'run', 
                                                     '--args', 'echo'], \
-                                        out_on=False, err_on=False)
-            using_gdb_wrapper = (ret == 0)
+                                        )#out_on=False, err_on=False)
+                using_gdb_wrapper = (ret == 0)
+                if not using_gdb_wrapper:
+                    logging.warning("use gdb is enabled but call to gdb fails"
+                                " (retcode {}) with msg: {}".format(ret, o_e))
+            else:
+                logging.warning("use gdb is enabled but gdb is not installed")
 
         crit_to_exes_map = {}
         obj = common_fs.loadJSON(self.instrumentation_details)
@@ -165,6 +207,12 @@ class CriteriaToolGCov(BaseCriteriaTool):
         exes_map = obj
         for criterion in enabled_criteria:
             crit_to_exes_map[criterion] = exes_map
+            
+        #logging.debug("DBG: {} {} {}".format(\
+        #                  "Using gdb wrapper is {}.".format(using_gdb_wrapper), \
+        #                  "crit_to_exe_map is {}.".format(crit_to_exes_map), \
+        #                  "wraper path is {}!".format(self.gcov_gdb_wrapper_sh)))
+            
         return crit_to_exes_map
     #~ def get_instrumented_executable_paths_map()
 
@@ -230,26 +278,49 @@ class CriteriaToolGCov(BaseCriteriaTool):
             os.chdir(self.gc_files_dir)
 
             # collect gcda (gcno)
-            r, _, _ = DriversUtils.execute_and_get_retcode_out_err(prog=prog, \
+            r, _, err_str = DriversUtils.execute_and_get_retcode_out_err(\
+                                        prog=prog, \
                                         args_list=args_list, out_on=False, \
-                                                                err_on=False)
+                                        err_on=True, merge_err_to_out=False)
 
             os.chdir(cwd)
             
-            if r != 0:
+            if r != 0: # or err_str:
                 ERROR_HANDLER.error_exit("Program {} {}.".format(prog,\
-                        'error collecting coverage is problematic'), __file__)
+                        'collecting coverage is problematic. ')+
+                        "The error msg is {}. \nThe command:\n{}".format(\
+                                err_str, " ".join([prog]+args_list)), __file__)
             
+            dot_gcov_file_list = self._get_gcov_list()
+            # Sources of interest
+            _, src_map = self.code_builds_factory.repository_manager.\
+                                                    get_relative_exe_path_map()
+            base_dot_gcov = [os.path.basename(f) for f in dot_gcov_file_list]
+            base_raw = [os.path.basename(f) for f in raw_filename_list]
+            interest = [os.path.basename(s) for s,o in src_map.items()]
+            interest = [s+'.gcov' for s in interest if os.path.splitext(s)[0] \
+                                                                in base_raw]
+            missed = set(interest) - set(base_dot_gcov)
+
+            # FIXME: Check for partial failure (compare number of gcno and gcov)
+            ERROR_HANDLER.assert_true(len(missed) == 0,
+                    "{} did not generate the '.gcov' files {}.".format(\
+                        prog, missed)+ "\nIts stderr is {}.\n CMD: {}".format(\
+                            err_str, " ".join([prog]+args_list)), __file__)
+
             # delete gcda
             for gcda_f in gcda_files:
                 os.remove(gcda_f)
-            
-        common_fs.dumpJSON(self._get_gcov_list(), \
+        else:
+            if not self.driver_config.get_allow_missing_coverage():
+                ERROR_HANDLER.error_exit(\
+                    "Testcase '{}' did not generate gcda, {}".format(\
+                        testcase, "when allow missing coverage is disabled"))
+            dot_gcov_file_list = []
+
+        common_fs.dumpJSON(dot_gcov_file_list, \
                                 os.path.join(result_dir_tmp,\
                                                 self.gcov_files_list_filename))
-        #else:
-        #    ERROR_HANDLER.error_exit(\
-        #                "Testcase '{}' did not generate gcda".format(testcase))
     #~ def _collect_temporary_coverage_data()
 
     def _extract_coverage_data_of_a_test(self, enabled_criteria, \
@@ -276,7 +347,9 @@ class CriteriaToolGCov(BaseCriteriaTool):
         # Sources of interest
         _, src_map = self.code_builds_factory.repository_manager.\
                                                     get_relative_exe_path_map()
-
+        
+        #logging.debug("gcov_list: {}".format(gcov_list))
+        
         for gcov_file in gcov_list:
             with open(gcov_file) as fp:
                 last_line = None
@@ -288,10 +361,26 @@ class CriteriaToolGCov(BaseCriteriaTool):
                     if len(col_split) > 2 and col_split[1] == '0':
                         # preamble
                         if col_split[2] == "Source":
-                            src_file = col_split[3]
-                            if src_file not in src_map:
-                                # src not in considered
-                                break
+                            src_file = os.path.normpath(col_split[3])
+                            if not src_file in src_map:
+                                s_cand = None
+                                for s in src_map.keys():
+                                    if s.endswith(os.sep+src_file):
+                                        ERROR_HANDLER.assert_true(s_cand is None,\
+                                              "multiple candidate, maybe same "+\
+                                              "source name in different dirs "+\
+                                              "for source {}".format(src_file), \
+                                                                        __file__)
+                                        s_cand = s
+                                if s_cand is None:
+                                    # If src file does not represent (not equal
+                                    # and not relatively equal, for case where
+                                    # build happens not in rootdir),
+                                    # src not in considered
+                                    break
+                                else:
+                                    # ensure compatibility in matrices
+                                    src_file = s_cand
                     elif line.startswith("function "):
                         # match function
                         parts = line.split()
@@ -326,6 +415,8 @@ class CriteriaToolGCov(BaseCriteriaTool):
         for gcov_f in self._get_gcov_list():
             os.remove(gcov_f)
 
+        #logging.debug("gcov res is: {}".format(res))
+        
         return res
     #~ def _extract_coverage_data_of_a_test()
 
@@ -354,10 +445,13 @@ class CriteriaToolGCov(BaseCriteriaTool):
 
         flags = ['-g', '--coverage', '-fprofile-dir='+self.gc_files_dir, '-O0']
         additionals = ["-fkeep-inline-functions"]
+        #additionals.append('-fprofile-abs-path')
+        #additionals.append('-fprofile-prefix-path='+code_builds_factory\
+        #                        .repository_manager.get_repository_dir_path())
         
         # get gcc version
         ret, out, err = DriversUtils.execute_and_get_retcode_out_err(\
-                                                        prog, ['-dumpversion'])
+                                            prog, args_list=['-dumpversion'])
         ERROR_HANDLER.assert_true(ret == 0, "'gcc -dumpversion' failed'")
         
         # if version > 6.5
