@@ -144,6 +144,8 @@ class MetaTestcaseTool(object):
                                                             "_checkpoints_")
         self.testcases_info_file = \
                 os.path.join(self.tests_working_dir, "testcasesInfos.json")
+        self.duplicated_tests_info_file = os.path.join(self.tests_working_dir,\
+                                            "duplicatedTestcasesInfos.json")
         self.flakiness_workdir = \
                             os.path.join(self.tests_working_dir, "Flakiness")
 
@@ -164,6 +166,12 @@ class MetaTestcaseTool(object):
         
         self.custom_devtest_toolalias = None
 
+        # Keep the duplicate test so that they are executed only once
+        # TODO: load from file(above), if not compute and store (in different func). in runtests, check that tests are here
+        # make sure that every test is is present, if not recompute.
+        self.tests_duplicates_map = None
+        self._update_tests_duplicates_map()
+
         # Create the diffent tools
         for idx in range(len(test_tool_config_list)):
             config = test_tool_config_list[idx]
@@ -235,6 +243,34 @@ class MetaTestcaseTool(object):
                             str(non_installed)))
     #~ def check_tools_installed()
 
+    def _update_tests_duplicates_map(self, recompute=False):
+        if os.path.isfile(self.duplicated_tests_info_file) and not recompute:
+            self.tests_duplicates_map = common_fs.loadJSON(self.duplicated_tests_info_file)
+            return 
+
+        groups = {}
+        for _, dat in self.testcases_configured_tools.items():
+            tobj = dat[self.TOOL_OBJ_KEY]
+            fmt = tobj.get_test_format_class()
+            if fmt is None:
+                continue
+            if fmt not in groups:
+                groups[fmt] = []
+            groups[fmt].append(tobj)
+
+        self.tests_duplicates_map = {}
+        # for each group do fdupes and merge all
+        for fmt, tt_list in groups.items():
+            if len(tt_list) > 1:
+                kepttest2duptest_map, test2keptdup = \
+                                                fmt.cross_tool_fdupes(*tt_list)
+                self.tests_duplicates_map.update(test2keptdup)
+        
+        if os.path.isdir(os.path.dirname(self.duplicated_tests_info_file)):
+            common_fs.dumpJSON(self.tests_duplicates_map, \
+                                self.duplicated_tests_info_file, pretty=True)
+    #~ def _update_tests_duplicates_map()
+
     def get_devtest_toolalias(self):
         return self.custom_devtest_toolalias
     #~ def get_devtest_toolalias()
@@ -422,6 +458,17 @@ class MetaTestcaseTool(object):
                                                 len(set(meta_testcases)), \
                                         "not all tests are unique", __file__)
 
+        # For fdupes
+        if len(self.tests_duplicates_map) > 0:
+            meta_testcases_backup = meta_testcases
+            meta_testcases = set(meta_testcases)
+            dups_remove_meta_testcases = meta_testcases & \
+                                                set(self.tests_duplicates_map)
+            dup_toadd_test = {self.tests_duplicates_map[v] for v in \
+                                dups_remove_meta_testcases} - meta_testcases
+            meta_testcases = (meta_testcases - dups_remove_meta_testcases) \
+                                                            | dup_toadd_test
+
         testcases_by_tool = {}
         for meta_testcase in meta_testcases:
             ttoolalias, testcase = \
@@ -504,8 +551,8 @@ class MetaTestcaseTool(object):
             
                 used = 0
                 for tt in para_tools:
-                    quota = int(len(testcases_by_tool[tt]) * sub_parallel_count / \
-                                                            para_tools_n_tests)
+                    quota = int(len(testcases_by_tool[tt]) * \
+                                    sub_parallel_count / para_tools_n_tests)
                     parallel_test_count_by_tool[tt] += quota
                     used += quota
                 for tt in para_tools:
@@ -612,6 +659,19 @@ class MetaTestcaseTool(object):
                                     set(meta_testcases)), \
                                                                      __file__)
 
+        # For fdupes
+        if len(self.tests_duplicates_map) > 0:
+            meta_testcases = meta_testcases_backup
+            for i in (0,1):
+                for mtest in dups_remove_meta_testcases:
+                    # add to results
+                    meta_test_failedverdicts_outlog[i][mtest] = copy.deepcopy(\
+                                        meta_test_failedverdicts_outlog[i]\
+                                            [self.tests_duplicates_map[mtest]])
+                for mtest in dup_toadd_test:
+                    # remove from results
+                    del meta_test_failedverdicts_outlog[i][mtest]
+
         if fault_test_execution_matrix_file is not None:
             # Load or Create the matrix 
             fault_test_execution_matrix = common_matrices.ExecutionMatrix( \
@@ -751,7 +811,7 @@ class MetaTestcaseTool(object):
         cp_func_name = "generate_tests"
         if test_tool_type_list is not None:
             for test_tool_type in sorted(test_tool_type_list, \
-													key=lambda x: x.get_str()):
+						key=lambda x: x.get_str()):
                 cp_func_name += ":" + test_tool_type.get_str()
         cp_task_id = 1
         checkpoint_handler = CheckPointHandler(\
@@ -790,6 +850,7 @@ class MetaTestcaseTool(object):
 
         # Invalidate any existing testcase info so it can be recomputed
         self._invalidate_testcase_info()
+        self._update_tests_duplicates_map(recompute=True)
 
         # @Checkpoint: Finished
         detailed_exectime = {}
@@ -899,7 +960,8 @@ class MetaTestcaseTool(object):
         return self.flakiness_workdir
     #~ def get_flakiness_workdir()
 
-    def check_get_flakiness(self, meta_testcases, repeat_count=2):
+    def check_get_flakiness(self, meta_testcases, repeat_count=2, \
+                                              get_flaky_tests_outputs=True):
         """
             Check if tests have flakiness by running multiple times
             :return: The list of flaky tests
@@ -952,7 +1014,7 @@ class MetaTestcaseTool(object):
                 os.remove(f)
 
         # get flaky tests outputs
-        if len(flaky_tests) > 0:
+        if get_flaky_tests_outputs and len(flaky_tests) > 0:
             for rep in range(repeat_count):
                 run(rep, list(flaky_tests), False)
             flaky_test_list_file = os.path.join(self.flakiness_workdir, \
